﻿#!/usr/bin/env python

# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
# -*- coding: utf-8 -*-
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)

__license__   = 'GPL v3'
__copyright__ = '2011, Pr.BarnArt, based on the Barnes work by Grant Drake'
__docformat__ = 'restructuredtext en'

import six
from six.moves import zip

import socket, re, datetime, string
from collections import OrderedDict
from threading import Thread
from lxml.html import fromstring, tostring
from xml.etree import ElementTree

from calibre.ebooks.metadata.book.base import Metadata
from calibre.library.comments import sanitize_comments_html
from calibre.utils.cleantext import clean_ascii_chars
from calibre.utils.localization import canonicalize_lang

import calibre_plugins.BOL_DE.config as cfg

class Worker(Thread): # Get  details
    '''
    Get book details from BOL.com book page in a separate thread
    related to BOL_DE
    '''
    name                    = 'Worker'
    description             = _('Get book details from BOL.de book page in a separate thread')
    author                  = 'Pr. BarnArt'
    version                 = (0, 9, 2)
    minimum_calibre_version = (0, 8, 0)
    

    BASE_URL_ISBN='http://www.bol.de/shop/buecher/suche/?fi=&st=&sa=&sr=&sv=&svb=&ssw=&si=&sk=&sd=&sre=&sq='
    BASE_ISBN= '&forward=weiter&sswg=ANY'
    dict_month={u'januar':1,u'februar':2,'märch':3,u'april':4,u'mai':5,u'juni':6,u'juli':7,u'august':8,u'september':9,u'oktober':10,u'october':10,u'november':11,u'dezember':12}


    def __init__(self, combo, result_queue, browser, log, relevance, plugin, timeout=20):      
        Thread.__init__(self)
        self.daemon = True
        cat1= 'Fotografie & Bildbände'
        #distract from combo
        self.url = combo[0]
        self.rating = combo[1]
        self.result_queue =  result_queue
        self.log, self.timeout = log, timeout
        self.relevance, self.plugin = relevance, plugin
        self.browser = browser.clone_browser()
        self.cover_url = self.bol_de_id = self.isbn = None
        self.cat_list = list()
        self.cat_list.append('Comics & Manga')
        self.cat_list.append('Allgemeines zu Comics & Manga')
        self.cat_list.append('Cartoons & Comics')
        self.cat_list.append('Graphic Novels')
        self.cat_list.append('Manga')
        self.cat_list.append('Erotik')
        self.cat_list.append('Comics & Mangas')
        self.cat_list.append('Fotografie & Bildbände')
        self.cat_list.append('Romane & Erzählungen')
        self.cat_list.append('Sex-Ratgeber')
        self.cat_list.append('Fachbächer')
        self.cat_list.append('Architektur')
        self.cat_list.append('Biowissenschaften')
        self.cat_list.append('Chemie')
        self.cat_list.append('Geowissenschaften')
        self.cat_list.append('Geschichtswissenschaft')
        self.cat_list.append('Informatik')
        self.cat_list.append('Ingenieurwissenschaften')
        self.cat_list.append('Kunstwissenschaft')
        self.cat_list.append('Mathematik')
        self.cat_list.append('Medienwissenschaft')
        self.cat_list.append('Medizin')
        self.cat_list.append('Musikwissenschaft')
        self.cat_list.append('Pädagogik')
        self.cat_list.append('Philosophie')
        self.cat_list.append('Physik & Astronomie')
        self.cat_list.append('Politikwissenschaft')
        self.cat_list.append('Psychologie')
        self.cat_list.append('Recht')
        self.cat_list.append('Sozialwissenschaft')
        self.cat_list.append('Sprach- & Literaturwissenschaft')
        self.cat_list.append('Theologie')
        self.cat_list.append('Wirtschaft')
        self.cat_list.append('Fantasy & Science Fiction')
        self.cat_list.append('Fantasy')
        self.cat_list.append('Science Fiction')
        self.cat_list.append('Vampirromane')
        self.cat_list.append('Freizeit & Hobby')
        self.cat_list.append('Basteln & Handarbeit')
        self.cat_list.append('Film & Fotografie')
        self.cat_list.append('Flug- & Fahrzeuge')
        self.cat_list.append('Haus & Garten')
        self.cat_list.append('Heimwerken & Do it yourself')
        self.cat_list.append('Kreatives Schreiben & Malen')
        self.cat_list.append('Musik & Instrumente')
        self.cat_list.append('Party & Dekoration')
        self.cat_list.append('Sammelleidenschaft')
        self.cat_list.append('Spiel & Spaß')
        self.cat_list.append('Sport & Fitness')
        self.cat_list.append('Jugendbücher')
        self.cat_list.append('Fantasy & Science Fiction')
        self.cat_list.append('Historische Romane')
        self.cat_list.append('Krimis & Thriller')
        self.cat_list.append('Preisgekrönte Jugendbücher')
        self.cat_list.append('Romane & Erzählungen')
        self.cat_list.append('Sachbücher')
        self.cat_list.append('Kochen & Backen')
        self.cat_list.append('Allgemeines zum Thema Kochen')
        self.cat_list.append('Alte Kochbücher')
        self.cat_list.append('Backen')
        self.cat_list.append('Biologisch kochen')
        self.cat_list.append('Desserts & Süßspeisen')
        self.cat_list.append('Diëten & Ernährungspläne')
        self.cat_list.append('Einmachen & Einkochen')
        self.cat_list.append('Feinschmecker &  Esskultur')
        self.cat_list.append('Getränke')
        self.cat_list.append('Gewürze & Kräuter')
        self.cat_list.append('Kochen für...')
        self.cat_list.append('Küche & Kochshows')
        self.cat_list.append('Paleo')
        self.cat_list.append('Saucen & Dips')
        self.cat_list.append('Tipps & Tricks')
        self.cat_list.append('Vegetarische & vegane Küche')
        self.cat_list.append('Vollwertküche')
        self.cat_list.append('Vorspeisen &  Salate')
        self.cat_list.append('Krimis & Thriller')
        self.cat_list.append('Gerichtsmedizin')
        self.cat_list.append('Historische Krimis')
        self.cat_list.append('Horror')
        self.cat_list.append('Preisgekrönte Krimis')
        self.cat_list.append('Regionalkrimis')
        self.cat_list.append('Thriller')
        self.cat_list.append('Tierkrimis')
        self.cat_list.append('Wahre Kriminalfälle')
        self.cat_list.append('Ratgeber')
        self.cat_list.append('Einkaufsführer')
        self.cat_list.append('Eltern & Kinder')
        self.cat_list.append('Ernährung')
        self.cat_list.append('Esoterik')
        self.cat_list.append('Gesundheit')
        self.cat_list.append('Kommunikation')
        self.cat_list.append('Lebensführung')
        self.cat_list.append('Lesben & Schwule')
        self.cat_list.append('Partnerschaft & Beziehungen')
        self.cat_list.append('Psychologie & Hilfe')
        self.cat_list.append('Rechts- & Finanzratgeber')
        self.cat_list.append('Weitere Ratgeber')
        self.cat_list.append('Reise & Abenteuer')
        self.cat_list.append('Abenteuer & Reiseberichte')
        self.cat_list.append('Aktivurlaub & Camping')
        self.cat_list.append('Atlanten & Karten')
        self.cat_list.append('Bildbände')
        self.cat_list.append('Reisefotografie')
        self.cat_list.append('Reiseführer')
        self.cat_list.append('Restaurant- & Hotelguides')
        self.cat_list.append('Urlaub mit der Familie')
        self.cat_list.append('Wandern & Radwandern')
        self.cat_list.append('Romane & Erzählungen')
        self.cat_list.append('Biografien & Tagebücher')
        self.cat_list.append('Dramatik')
        self.cat_list.append('Historische Romane')
        self.cat_list.append('Klassiker')
        self.cat_list.append('Kurzgeschichten & Anthologien')
        self.cat_list.append('Liebesromane')
        self.cat_list.append('Lyrik')
        self.cat_list.append('Preisgekrönte Romane')
        self.cat_list.append('Romane & Erzählungen')
        self.cat_list.append('Unterhaltung für Frauen')
        self.cat_list.append('Witz & Unterhaltung')
        self.cat_list.append('Sachbücher')
        self.cat_list.append('Biografien & Erinnerungen')
        self.cat_list.append('Business & Karriere')
        self.cat_list.append('Computer & Internet')
        self.cat_list.append('Esoterik')
        self.cat_list.append('Kunst & Kultur')
        self.cat_list.append('Naturwissenschaften & Technik')
        self.cat_list.append('Politik & Geschichte')
        self.cat_list.append('Religion & Glaube')
        self.cat_list.append('Schule & Lernen')
        self.cat_list.append('Formelsammlungen')
        self.cat_list.append('Lehrermaterialien')
        self.cat_list.append('Lektüren & Interpretationen')
        self.cat_list.append('Lernhilfen')
        self.cat_list.append('Lexika & Wörterbücher')
        self.cat_list.append('Schulbücher')
        self.cat_list.append('Schultüten basteln')
        self.cat_list.append('Sprachkurse & Sprachtrainer')
        
        self.droot = None
        lm = {
                'eng': ('English', 'Englisch'),
                'fra': ('French', 'Français'),
                'ita': ('Italian', 'Italiano'),
                'dut': ('Dutch','Nederlands'),
                'deu': ('German', 'Deutsch','Duits','Deutsch, Schwedisch'),
                'spa': ('Spanish', 'Espa\xf1ol', 'Espaniol'),
                'jpn': ('Japanese', u'日本語'),
                'por': ('Portuguese', 'Português'),
                }
        self.lang_map = {}
        for code, names in lm.items():
            for name in names:
                self.lang_map[name] = code
                
    def unescape(self,html_txt):
        try:
            import html
            return html.unescape(html_txt)
        except ImportError:
            from HTMLParser import HTMLParser   
            return HTMLParser().unescape(html_txt)
            
    def run(self):
        try:
            self.get_details()
        except:
            self.log.exception('get_details failed for url: %r'%self.url)

    def get_details(self):
        try:
            self.log.info('BOL_DE url: %r'%self.url)
            raw = self.browser.open_novisit(self.url, timeout=self.timeout).read().strip()
        except Exception as e:
            if callable(getattr(e, 'getcode', None)) and \
                    e.getcode() == 404:
                self.log.error('URL malformed: %r'%self.url)
                return
            attr = getattr(e, 'args', [None])
            attr = attr if attr else [None]
            if isinstance(attr[0], socket.timeout):
                msg = 'Bol.com timed out. Try again later.'
                self.log.error(msg)
            else:
                msg = 'Failed to make details query: %r'%self.url
                self.log.exception(msg)
            return
        #raw = raw.decode('windows-1252', errors='replace')
        raw = raw.decode('utf-8', errors='replace')
        if '<title>404 - ' in raw:
            self.log.error('URL malformed: %r'%self.url)
            return
        try:
            root = fromstring(clean_ascii_chars(raw))           
        except:
            msg = 'Failed to parse Bol.dedetails page: %r'%self.url
            self.log.exception(msg)
            return
        self.parse_details(root,raw)

    def parse_details(self, root,raw):  
        try:
            bol_de_id = self.parse_isbn(root)
            if  bol_de_id:
                bol_de_id=bol_de_id.replace('-','') 
        except:
            self.log.exception('Error parsing BOL_DE id for url: %r'%self.url)
            bol_de_id = None
        try:    
            title = self.parse_title(root)
        except:
            self.log.exception('Error parsing title for url: %r'%self.url)
            title = None
        try:
            authors = self.parse_authors(root)
        except:
            self.log.exception('Error parsing authors for url: %r'%self.url)
            authors = []            
        if not title or not authors:
            self.log.error('Could not find title/authors/bol_de_id for %r'%self.url)
            self.log.error('Title: %r Authors: %r'%( title, authors))       
            return
        try:
            series = None
            series_index = None
            series,series_index=self.parse_series(root)
        except:
            self.log.exception('Error parsing series for url: %r'%self.url)    
            series = None
            series_index = None 
        mi = Metadata(title, authors)
        if series:
            mi.series = series
            mi.series_index = series_index
        self.bol_de_id = bol_de_id
        try:
            mi.language=self.parse_language(root)
        except:
            self.log.exception('Error parsing ISBN for url: %r'%self.url)  
        try:
            isbn = self.parse_isbn(root)
            print(('isbn:', isbn))
            if isbn:
                self.isbn = isbn.replace('-','')
                mi.isbn = isbn
        except:
            self.log.exception('Error parsing ISBN for url: %r'%self.url)  
        try:
            mi.rating=self.parse_rating_d(root)
            print(('rating:',mi.rating))
        except:
            self.log.exception('Error parsing rating')
        try:
            mi.comments = self.parse_comments_d(root)
            #print(('comments:', mi.comments))
        except:
            self.log.exception('Error parsing comments for url: %r'%self.url)
        try:
            self.cover_url = self.parse_cover(root,raw)
        except:
            self.log.exception('Error parsing cover for url: %r'%self.url)
        mi.has_cover = bool(self.cover_url)
        try:
            tags = self.parse_tags(root)
            if tags:
                mi.tags = tags
        except:
            self.log.exception('Error parsing tags for url: %r'%self.url)
        try:
            mi.publisher = self.parse_publisher_d(root) 
        except:
            self.log.exception('Error parsing publisher for url: %r'%self.url)
        try:
            tpub=self.parse_published_date_d(root)
            print(('tpub:',tpub))
            mi.pubdate=tpub
        except:
            self.log.exception('Error parsing published date for url: %r'%self.url)
        mi.source_relevance = self.relevance    
        if self.bol_de_id:
            if self.isbn:
                self.plugin.cache_isbn_to_identifier(self.isbn, self.bol_de_id)
        self.plugin.clean_downloaded_metadata(mi)
        mi.title=title
        self.result_queue.put(mi)

    def parse_bol_de_id(self, url):
        return 1

    def parse_title(self,root):
        title_node=root.xpath('//section[2][@class="artikel-infos" ]/h1 [@class="ncTitle"]')
        print(('title_node:', title_node))
        if title_node:
            title=title_node[0].text_content().strip()
            title=title.replace('\n','').strip()
            pos=title.find(': ')
            if pos>0:
                title= title[pos+2:]
            pos=title.find(' /')
            print('title1:', title)
            if pos>0:
                title= title[:pos]
            print('title2:', title)
            return title
    
    def parse_authors(self, root): 
        authors_node=root.xpath('//section[2][@class="artikel-infos" ]/p [@class="aim-author"]/a')    
        print(('authors_node:',authors_node))
        authors = []
        if authors_node:
            for auteurs in authors_node:
                auteur=ElementTree.tostring(auteurs, method='text',encoding='utf-8').decode()
                
                print(('auteur:', auteur))
                auteur= self.unescape(auteur)
                if auteur not in authors:
                    authors.append(auteur)
                    #print(('authors:', authors))
        return authors

    def parse_series(self,root):
        series = None
        index = None
        serie_node=root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"Reihe")]]/../td')
        if serie_node:    
            serie=serie_node[0].text_content().strip()
            pos=serie.rfind(' ')
            if pos>0:
                series=serie[0:pos]
                index=serie[pos+1:]
                print('s,nr:',series, index)
                self.log.info('s,nr:',series, index)
                return(series,index)
        serie_node=root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"Reihe")]]/../td')
        if serie_node:    
            serie=serie_node[0].text_content().strip()
            pos=serie.rfind(' ')
            if pos>0:
                series=serie[0:pos]
                index=serie[pos+1:]
                print('s,nr:',series, index)
                self.log.info('s,nr:',series, index)
                return(series,index)
        serie_node=root.xpath('//section[2][@class="artikel-infos" ]/p[@class="ncSerie"]/a')
        print('serie_node_3:',serie_node)
        if serie_node:    
            serie=serie_node[0].text_content().strip()
            pos=serie.rfind(' ')
            if pos>0:
                series=serie[0:pos]
                index=serie[pos+1:]
                print('s,nr:',series, index)
                pos=series.rfind('Band')
                if pos>1:
                    series=serie[0:pos-1]
                self.log.info('s,nr:',series, index)
                return(series,index)
        return(series,index)

    def parse_isbn(self, root):
        ean_node=root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"EAN")]]/../td')
        print('ean_node 2 rij:',ean_node)
        if ean_node:    
            ean=ean_node[0].text_content().strip()
            return ean
        isbn_node=root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"ISBN")]]/../td')
        print('isbn_node 2 rij:',isbn_node)
        if isbn_node:    
            isbn=isbn_node[0].text_content().strip()
            return isbn
        ean_node=root.xpath('//div[@class="small-12 large-6 columns first"]/table/tbody/tr/th [text()[contains(.,"EAN")]]/../td')
        print('ean_node 1e rij:',ean_node)
        if ean_node:    
            ean=ean_node[0].text_content().strip()
            return ean
        isbn_node=root.xpath('//div[@class="small-12 large-6 columns first"]/table/tbody/tr/th [text()[contains(.,"ISBN")]]/../td')
        print('isbn_node 1 rij:',isbn_node)
        if isbn_node:    
            isbn=isbn_node[0].text_content().strip()
            return isbn


    def parse_language(self,root):
        def set_ans(node):
            t_ans = None
            self.log.info("_parse_language: Have language node")
            raw1 = node[0].text_content().strip()
            self.log.info("_parse_language: raw1=", raw1)
            t_ans = self.lang_map.get(raw1, None)
            if t_ans:
                self.log.info("_parse_language: ans=", t_ans)
                return t_ans
            t_ans = canonicalize_lang(raw1)
            if t_ans:
                self.log.info("_parse_language cano: ans=", t_ans)
                return t_ans
            return(None)
        lang='Deutsch'
        ans = None
        lang_node2=root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"Sprache")]]/../td')
        if lang_node2:
            ans=set_ans(lang_node2)
        lang_node1=root.xpath('//div[@class="small-12 large-6 columns first"]/table/tbody/tr/th [text()[contains(.,"Sprache")]]/../td')
        if lang_node1:
            ans=set_ans(lang_node1)
        return ans

    def parse_rating_d(self, root):
        rating_node = root.xpath('//section[@class="artikel-infos" ]/a[@class="sterne"]/div/div/span [@class="active rating-star"]')
        print ('rating_node:', rating_node)
        rat = None
        if rating_node:
            rat=len(list(rating_node))
        return rat

    def parse_tags(self, root): 
        tags_nodes=root.xpath('//div [@class="bd bd_artikeldetails clearfix"]/span/a')
        tags_nodes=root.xpath('//ul [@class="breadcrumbs"]/li/a')
        if tags_nodes:
            calibre_tags= list()
            for t_node in tags_nodes:
                txt=t_node.text.strip()
                if txt in self.cat_list:
                    if txt not in calibre_tags:
                        calibre_tags.append(txt)
            if len(calibre_tags) > 0:
                return calibre_tags

    def parse_publisher_d(self, root):
        publisher = None
        def set_publisher(node):
            t_publisher= None
            t_publisher=node[0].text_content().strip()
            pos=t_publisher.find('...')
            if pos>0:
                pub2_node=root.xpath('//div [@id="b9_featurelist"]/div/div[2]/ul/li/span[@class="value"]/a/@href')
                if pub2_node:           
                    publisher2=pub2_node[0]
                    pos=publisher2.rfind('/')
                    publisher2=publisher2[pos+1:]
                    pos=publisher2.find('.html')
                    if pos>0:
                        publisher2=publisher2[:pos]
                    publisher2=publisher2.replace('e_books','e-books')
                    publisher2=publisher2.replace('_',' ');
                    publisher2=string.capwords(publisher2)
                    publisher2=publisher2.replace('Der','der')
                    t_publisher=publisher2
            return (t_publisher)
        pub_node1 = root.xpath('//div[@class="small-12 large-6 columns first"]/table/tbody/tr/th [text()[contains(.,"Verlag")]]/../td/a')
        if pub_node1:
            publisher=set_publisher(pub_node1)
        pub_node2 = root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"Verlag")]]/../td/a')
        if pub_node2:
            publisher=set_publisher(pub_node2)                    
        return(publisher)
    
    def parse_published_date_d(self, root):
        pub_date = None 
        def set_pubdate(node):
            t_pub_date= None
            t_pub_date=node[0].text_content().strip()
            print(('t_pub_date:', t_pub_date))
            return (t_pub_date)
        pub_node1 = root.xpath('//div[@class="small-12 large-6 columns first"]/table/tbody/tr/th [text()[contains(.,"Erscheinungsdatum")]]/../td')
        if pub_node1:
            pub_date=set_pubdate(pub_node1)
        pub_node2 = root.xpath('//div[@class="small-12 large-6 columns second show-for-large"]/table/tbody/tr/th [text()[contains(.,"Erscheinungsdatum")]]/../td')
        if pub_node2:
            pub_date=set_pubdate(pub_node1)
        if pub_date:
            return self._convert_date_text_d(pub_date)
            

    def _convert_date_text_d(self, date_text):
        # Note that the date text could be "2003", "december 2003" or "December 10th 2003"\
        #print(('date_text:', date_text))
        year = int(date_text[-4:])
        month = 1
        day = 1
        pos = date_text.find('.')
        if pos>0:   
            if len(date_text) > 4:
                text_parts = date_text[:len(date_text)-5].partition('.')
                if len(text_parts[2])>0:
                    month = int(re.match('([0-9]+)', text_parts[2]).groups(0)[0])
                if len(text_parts[0]) > 0:
                    day = int(re.match('([0-9]+)', text_parts[0]).groups(0)[0])
        else:
            text_parts = date_text[:len(date_text)-5].partition(' ')
            print(('text_parts:',text_parts))
            for  txt in text_parts:
                for key  in self.dict_month:
                    if txt.lower() ==key:
                        month= self.dict_month[key]
                        break;
        from calibre.utils.date import utc_tz
        print(('ymd:',year,month, day))
        print(('datum:',datetime.datetime(year, month, day, tzinfo=utc_tz)))
        return datetime.datetime(year, month, day, tzinfo=utc_tz)

    def parse_comments_d(self, root):
        comments = ''
        description_node = root.xpath('//section[@class="artikel-infos" ]/div[@class="beschreibung"]/div[@class="text-infos"]/p')
        print(('description_node:',description_node))
        if description_node:
            for node in description_node:
                comments =comments +ElementTree.tostring(node, method="text",encoding='utf-8').decode()
                #print(('comments:',comments))
                #comments= description_node[0].text_content().strip()
                comments = comments.replace('<div class="redaktion raw">','').strip()
                comments=comments.replace('<h2>','')
                comments=comments.replace('<strong>','')
                comments=comments.replace('</strong>','<br>')
                comments=comments.replace('</h2>','<br>')
                comments =comments.replace('\n',' ')
                end=comments.find('<div class')
                if end>0:
                    comments=comments[0:end]
        if comments:
            comments= comments + '<br>(source: Bol.de)<br>'
            return comments

    def parse_cover(self, root,raw):
        result_node=root.xpath('//section [@class="previews"]/ul[1]/li[@data-slide="prev-0"]/picture/img/@data-original')
        print('result_node cover:', result_node)
        if result_node:
            img_url = result_node[0]
            self.plugin.cache_identifier_to_cover_url(self.bol_de_id, img_url)
            return img_url
        result_node=root.xpath('//meta [@property="og:image"]/@content')
        #print(result_node)   
        if result_node:
            img_url = result_node[0]
            #print('img_url:',img_url)
            self.plugin.cache_identifier_to_cover_url(self.bol_de_id, img_url)
            return img_url
        
